function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

import * as t from '@babel/types';
import { isTypedVariable, OperationType } from 'graphql-tool-utilities';
import { ExportFormat } from "../../types.mjs";
import { Options, FileContext, OperationContext } from "./context.mjs";
import { ObjectStack } from "./utilities.mjs";
import { tsInterfaceBodyForObjectField, variablesInterface } from "./language.mjs";

var generate = require('@babel/generator')["default"];

export function printDocument(_ref, ast, options) {
  var path = _ref.path,
      operation = _ref.operation,
      fragments = _ref.fragments;
  var file = new FileContext(path, options);

  if (operation == null) {
    var _fileBody = fragments.reduce(function (statements, fragment) {
      var context = new OperationContext(fragment, ast, options, file);
      var body = tsInterfaceBodyForObjectField(fragment, fragment.typeCondition, new ObjectStack(fragment.typeCondition), context);
      var namespace = context.namespace;
      return [].concat(_toConsumableArray(statements), _toConsumableArray(namespace ? [t.exportNamedDeclaration(namespace, [])] : []), [t.exportNamedDeclaration(t.tsInterfaceDeclaration(t.identifier(context.typeName), null, null, body), [])]);
    }, []);

    var _schemaImports = file.schemaImports;

    if (_schemaImports) {
      _fileBody.unshift(_schemaImports);
    }

    return generate(t.file(t.program(_fileBody), [], [])).code;
  }

  var context = new OperationContext(operation, ast, options, file);
  var partialContext = new OperationContext(operation, ast, _objectSpread(_objectSpread({}, options), {}, {
    partial: true
  }), file);
  var rootType;

  if (operation.operationType === OperationType.Query) {
    rootType = ast.schema.getQueryType();
  } else if (operation.operationType === OperationType.Mutation) {
    rootType = ast.schema.getMutationType();
  } else {
    rootType = ast.schema.getSubscriptionType();
  }

  var variables = operation.variables.filter(isTypedVariable).length > 0 ? context["export"](variablesInterface(operation.variables, context)) : null;
  var operationInterface = t.tsInterfaceDeclaration(t.identifier(context.typeName), null, null, tsInterfaceBodyForObjectField(operation, rootType, new ObjectStack(rootType), context));
  var operationPartialInterface = t.tsInterfaceDeclaration(t.identifier(partialContext.typeName), null, null, tsInterfaceBodyForObjectField(operation, rootType, new ObjectStack(rootType), partialContext));
  var schemaImports = file.schemaImports;
  var namespace = context.namespace;
  var partialNamespace = partialContext.namespace;
  var _options$exportFormat = options.exportFormat,
      exportFormat = _options$exportFormat === void 0 ? ExportFormat.Document : _options$exportFormat;
  var documentType = exportFormat === ExportFormat.Document ? 'DocumentNode' : 'SimpleDocument';
  var documentNodeImport = t.importDeclaration([t.importSpecifier(t.identifier(documentType), t.identifier(documentType))], t.stringLiteral('graphql-typed'));
  var documentNodeDeclaratorIdentifier = t.identifier('document');
  documentNodeDeclaratorIdentifier.typeAnnotation = t.tsTypeAnnotation(t.tsTypeReference(t.identifier(documentType), t.tsTypeParameterInstantiation([t.tsTypeReference(t.identifier(context.typeName)), variables || t.tsNeverKeyword(), t.tsTypeReference(t.identifier(partialContext.typeName))])));
  var documentNodeDeclaration = t.variableDeclaration('const', [t.variableDeclarator(documentNodeDeclaratorIdentifier)]);
  documentNodeDeclaration.declare = true;
  var documentNodeExport = t.exportDefaultDeclaration(t.identifier('document'));
  var fileBody = [documentNodeImport];

  if (schemaImports) {
    fileBody.push(schemaImports);
  }

  if (partialNamespace) {
    fileBody.push(t.exportNamedDeclaration(partialNamespace, []));
  }

  fileBody.push(t.exportNamedDeclaration(operationPartialInterface, []));

  if (namespace) {
    fileBody.push(t.exportNamedDeclaration(namespace, []));
  }

  fileBody.push(t.exportNamedDeclaration(operationInterface, []), documentNodeDeclaration, documentNodeExport);
  return generate(t.file(t.program(fileBody), [], [])).code;
}