function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

import * as t from '@babel/types';
import { GraphQLString, isEnumType, isObjectType, isNonNullType, isScalarType, isListType, isAbstractType, GraphQLNonNull, isInputObjectType, isInterfaceType, isUnionType } from 'graphql';
import { isTypedVariable } from 'graphql-tool-utilities';
import { scalarTypeMap } from "../utilities.mjs";
export function tsInterfaceBodyForObjectField(_ref, graphQLType, stack, context) {
  var _ref$fields = _ref.fields,
      fields = _ref$fields === void 0 ? [] : _ref$fields;
  var requiresTypename = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;
  var schema = context.ast.schema;
  var isRootType = !Array.isArray(graphQLType) && (graphQLType === schema.getQueryType() || graphQLType === schema.getMutationType() || graphQLType === schema.getSubscriptionType());
  var uniqueFields = fields.filter(function (field) {
    if (stack.hasSeenField(field)) {
      return false;
    }

    stack.sawField(field);
    return true;
  });
  var typenameField = {
    fieldName: '__typename',
    responseName: '__typename',
    type: new GraphQLNonNull(GraphQLString),
    isConditional: false
  };
  var typename = (context.options.addTypename && !isRootType || requiresTypename) && !stack.hasSeenField(typenameField) ? tsPropertyForField(typenameField, graphQLType, stack, context, requiresTypename) : null;
  var body = uniqueFields.map(function (field) {
    return tsPropertyForField(field, graphQLType, stack, context, requiresTypename);
  });
  return t.tsInterfaceBody(typename ? [typename].concat(_toConsumableArray(body)) : body);
}

function tsTypeForInlineFragment(inlineFragment, _graphQLType, stack, context) {
  var requiresTypename = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;
  var typeCondition = inlineFragment.typeCondition;
  var interfaceDeclaration = t.tsInterfaceDeclaration(t.identifier(stack.name), null, null, tsInterfaceBodyForObjectField(inlineFragment, typeCondition, stack, context, requiresTypename));
  return context["export"](interfaceDeclaration);
}

function tsTypeForObjectField(field, graphQLType, stack, context) {
  var _field$inlineFragment = field.inlineFragments,
      inlineFragments = _field$inlineFragment === void 0 ? [] : _field$inlineFragment;
  var possibleTypes = isInterfaceType(graphQLType) || isUnionType(graphQLType) ? context.ast.schema.getPossibleTypes(graphQLType) : [];

  if (inlineFragments.length) {
    var fragmentTypes = _toConsumableArray(inlineFragments).map(function (inlineFragment) {
      return tsTypeForInlineFragment(inlineFragment, graphQLType, stack.fragment(inlineFragment.typeCondition), context, context.options.partial);
    });

    var typesCoveredByInlineFragments = new Set(_toConsumableArray(inlineFragments).reduce(function (types, inlineFragment) {
      return [].concat(_toConsumableArray(types), _toConsumableArray(inlineFragment.possibleTypes));
    }, []));
    var missingPossibleTypes = possibleTypes.filter(function (possibleType) {
      return !typesCoveredByInlineFragments.has(possibleType);
    });
    var otherStack = stack.fragment();
    var otherTypeInterface = t.tsInterfaceDeclaration(t.identifier(otherStack.name), null, null, tsInterfaceBodyForObjectField(field, missingPossibleTypes, otherStack, context, context.options.partial));
    var otherType = context["export"](otherTypeInterface);
    return t.tsUnionType([].concat(_toConsumableArray(fragmentTypes), [otherType]));
  } else if (possibleTypes.length === 1) {
    // When we have an interface or union type, but no inline fragments, it
    // means that there is only one conforming type for the union/ interface.
    // Here, we construct a "nothing" type that stands in for future additions
    // to the membership of the union/ interface.
    var _otherStack = stack.fragment();

    var _otherTypeInterface = t.tsInterfaceDeclaration(t.identifier(_otherStack.name), null, null, tsInterfaceBodyForObjectField({
      fields: []
    }, [], _otherStack, context, context.options.partial));

    var interfaceStack = stack.fragment(possibleTypes[0]);

    var _interfaceDeclaration = t.tsInterfaceDeclaration(t.identifier(interfaceStack.name), null, null, tsInterfaceBodyForObjectField(field, graphQLType, interfaceStack, context));

    return t.tsUnionType([context["export"](_interfaceDeclaration), context["export"](_otherTypeInterface)]);
  }

  var interfaceDeclaration = t.tsInterfaceDeclaration(t.identifier(stack.name), null, null, tsInterfaceBodyForObjectField(field, graphQLType, stack, context));
  return context["export"](interfaceDeclaration);
}

function tsTypenameForGraphQLType(type) {
  return t.tsLiteralType(t.stringLiteral(type.name));
}

function tsPropertyForField(field, parentType, stack, context) {
  var isRequiredTypename = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;

  if (field.fieldName === '__typename' && parentType) {
    var optional = !isRequiredTypename && (context.options.partial || field.isConditional || !isNonNullType(field.type));
    var parentTypes = Array.isArray(parentType) ? parentType : [parentType];
    var allPossibleTypes = parentTypes.reduce(function (all, type) {
      return [].concat(_toConsumableArray(all), _toConsumableArray(isAbstractType(type) ? context.ast.schema.getPossibleTypes(type) : [type]));
    }, []);
    var typename;

    if (allPossibleTypes.length === 0) {
      typename = t.tsLiteralType(t.stringLiteral(''));
    } else if (allPossibleTypes.length > 1) {
      typename = t.tsUnionType(allPossibleTypes.map(tsTypenameForGraphQLType));
    } else {
      typename = tsTypenameForGraphQLType(allPossibleTypes[0]);
    }

    var typenameProperty = t.tsPropertySignature(t.identifier(field.responseName), optional ? t.tsTypeAnnotation(t.tsUnionType([typename, t.tsNullKeyword()])) : t.tsTypeAnnotation(typename));
    typenameProperty.optional = optional;
    return typenameProperty;
  }

  var property = t.tsPropertySignature(t.identifier(field.responseName), t.tsTypeAnnotation(tsTypeForGraphQLType(field.type, field, stack, context)));
  property.optional = context.options.partial || field.isConditional || !isNonNullType(field.type);
  return property;
}

function tsTypeForGraphQLType(graphQLType, field, stack, context) {
  var type;
  var forceNullable = context.options.partial || field.isConditional && graphQLType === field.type;
  var isNonNull = !forceNullable && isNonNullType(graphQLType);
  var unwrappedGraphQLType = isNonNullType(graphQLType) ? graphQLType.ofType : graphQLType;

  if (isScalarType(unwrappedGraphQLType)) {
    if (Object.prototype.hasOwnProperty.call(scalarTypeMap, unwrappedGraphQLType.name)) {
      type = scalarTypeMap[unwrappedGraphQLType.name];
    } else {
      context.file["import"](unwrappedGraphQLType.name);
      type = t.tsTypeReference(t.identifier(unwrappedGraphQLType.name));
    }
  } else if (isEnumType(unwrappedGraphQLType)) {
    context.file["import"](unwrappedGraphQLType.name);
    type = t.tsTypeReference(t.identifier(unwrappedGraphQLType.name));
  } else if (isListType(unwrappedGraphQLType)) {
    var ofType = unwrappedGraphQLType.ofType;
    var arrayType = tsTypeForGraphQLType(ofType, field, stack, context);
    type = t.tsArrayType(t.isTSUnionType(arrayType) ? t.tsParenthesizedType(arrayType) : arrayType);
  } else if (isObjectType(unwrappedGraphQLType) || isInterfaceType(unwrappedGraphQLType) || isUnionType(unwrappedGraphQLType)) {
    type = tsTypeForObjectField(field, unwrappedGraphQLType, stack.nested(field, unwrappedGraphQLType), context);
  } else {
    type = t.tsAnyKeyword();
  }

  return isNonNull ? type : t.tsUnionType([type, t.tsNullKeyword()]);
}

export function variablesInterface(variables, context) {
  return t.tsInterfaceDeclaration(t.identifier('Variables'), null, null, t.tsInterfaceBody(variables.filter(isTypedVariable).map(function (variable) {
    return tsPropertyForVariable(variable, context);
  })));
}

function tsPropertyForVariable(_ref2, context) {
  var name = _ref2.name,
      type = _ref2.type;
  var property = t.tsPropertySignature(t.identifier(name), t.tsTypeAnnotation(tsTypeForGraphQLInputType(type, context)));
  property.optional = !isNonNullType(type);
  return property;
}

function tsTypeForGraphQLInputType(graphQLType, context) {
  var type;
  var unwrappedGraphQLType = isNonNullType(graphQLType) ? graphQLType.ofType : graphQLType;

  if (isScalarType(unwrappedGraphQLType)) {
    if (Object.prototype.hasOwnProperty.call(scalarTypeMap, unwrappedGraphQLType.name)) {
      type = scalarTypeMap[unwrappedGraphQLType.name];
    } else {
      context.file["import"](unwrappedGraphQLType.name);
      type = t.tsTypeReference(t.identifier(unwrappedGraphQLType.name));
    }
  } else if (isEnumType(unwrappedGraphQLType) || isInputObjectType(unwrappedGraphQLType)) {
    context.file["import"](unwrappedGraphQLType.name);
    type = t.tsTypeReference(t.identifier(unwrappedGraphQLType.name));
  } else {
    var ofType = unwrappedGraphQLType.ofType;
    var arrayType = tsTypeForGraphQLInputType(ofType, context);
    type = t.tsArrayType(isNonNullType(ofType) ? arrayType : t.tsParenthesizedType(arrayType));
  }

  return isNonNullType(graphQLType) ? type : t.tsUnionType([type, t.tsNullKeyword()]);
}