"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DefaultGraphQLFilesystem = void 0;

var _chokidar = require("chokidar");

var _graphqlConfigUtilities = require("graphql-config-utilities");

var _graphqlFilesystem = require("./graphql-filesystem.js");

class DefaultGraphQLFilesystem extends _graphqlFilesystem.AbstractGraphQLFilesystem {
  constructor(...args) {
    super(...args);
    this.watchers = [];
  }

  async watch(config) {
    this.watchers.push(...this.setupDocumentWatchers(config).concat(this.setupSchemaWatcher(config)));
    await Promise.all(this.watchers.map(watcher => new Promise(resolve => watcher.on('ready', () => resolve()))));
  }

  dispose() {
    this.watchers.forEach(watcher => {
      watcher.close();
    });
    this.watchers.length = 0;
  }

  getGraphQLProjectIncludedFilePaths(projectConfig) {
    return (0, _graphqlConfigUtilities.getGraphQLProjectIncludedFilePaths)(projectConfig);
  }

  setupDocumentWatchers(config) {
    return (0, _graphqlConfigUtilities.getGraphQLProjects)(config).filter(({
      includes
    }) => includes.length > 0).map(projectConfig => {
      return (0, _chokidar.watch)(projectConfig.includes.map(include => (0, _graphqlConfigUtilities.resolvePathRelativeToConfig)(projectConfig, include)), {
        ignored: projectConfig.excludes.map(exclude => (0, _graphqlConfigUtilities.resolvePathRelativeToConfig)(projectConfig, exclude)),
        ignoreInitial: true
      }).on('add', filePath => this.emit('change:document', filePath, projectConfig)).on('change', filePath => this.emit('change:document', filePath, projectConfig)).on('unlink', filePath => this.emit('delete:document', filePath, projectConfig));
    });
  }

  setupSchemaWatcher(config) {
    return (0, _chokidar.watch)((0, _graphqlConfigUtilities.getGraphQLSchemaPaths)(config), {
      ignoreInitial: true
    }).on('change', schemaPath => this.emit('change:schema', schemaPath));
  }

}

exports.DefaultGraphQLFilesystem = DefaultGraphQLFilesystem;