"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.OperationContext = exports.FileContext = void 0;

var _path = require("path");

var t = _interopRequireWildcard(require("@babel/types"));

var _upperCaseFirst = require("upper-case-first");

var _graphqlToolUtilities = require("graphql-tool-utilities");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

class FileContext {
  get schemaImports() {
    const {
      path,
      importedTypes,
      options: {
        schemaTypesPath
      }
    } = this;
    return importedTypes.size > 0 ? t.importDeclaration([...importedTypes].map(type => t.importSpecifier(t.identifier(type), t.identifier(type))), t.stringLiteral(importPath(path, schemaTypesPath))) : null;
  }

  constructor(path, options) {
    this.path = path;
    this.options = options;
    this.importedTypes = new Set();
  }

  import(type) {
    this.importedTypes.add(type);
  }

}

exports.FileContext = FileContext;

class OperationContext {
  get typeName() {
    let typeName;

    if ((0, _graphqlToolUtilities.isOperation)(this.operation)) {
      const {
        operationName,
        operationType
      } = this.operation;
      typeName = `${(0, _upperCaseFirst.upperCaseFirst)(operationName)}${(0, _upperCaseFirst.upperCaseFirst)(operationType)}Data`;
    } else {
      const {
        fragmentName
      } = this.operation;
      typeName = `${(0, _upperCaseFirst.upperCaseFirst)(fragmentName)}FragmentData`;
    }

    return this.options.partial ? typeName.replace(/Data$/, 'PartialData') : typeName;
  }

  get namespace() {
    const {
      exported,
      typeName
    } = this;
    return exported.length > 0 ? t.tsModuleDeclaration(t.identifier(typeName), t.tsModuleBlock(exported.map(type => t.exportNamedDeclaration(type, [])))) : null;
  }

  get exported() {
    return this.exportedTypes;
  }

  constructor(operation, ast, options, file) {
    this.operation = operation;
    this.ast = ast;
    this.options = options;
    this.file = file;
    this.exportedTypes = [];
  }

  export(type) {
    this.exportedTypes.push(type);
    return t.tsTypeReference(t.tsQualifiedName(t.identifier(this.typeName), t.identifier(type.id.name)));
  }

}

exports.OperationContext = OperationContext;

function importPath(from, to) {
  const relativePath = (0, _path.relative)((0, _path.dirname)(from), to);
  const normalizedPath = relativePath.startsWith('..') ? relativePath : `./${relativePath}`;
  return normalizedPath.replace(/\.ts$/, '');
}