"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.printDocument = printDocument;

var t = _interopRequireWildcard(require("@babel/types"));

var _graphqlToolUtilities = require("graphql-tool-utilities");

var _types2 = require("../../types.js");

var _context = require("./context.js");

var _utilities = require("./utilities.js");

var _language = require("./language.js");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const generate = require('@babel/generator').default;

function printDocument({
  path,
  operation,
  fragments
}, ast, options) {
  const file = new _context.FileContext(path, options);

  if (operation == null) {
    const fileBody = fragments.reduce((statements, fragment) => {
      const context = new _context.OperationContext(fragment, ast, options, file);
      const body = (0, _language.tsInterfaceBodyForObjectField)(fragment, fragment.typeCondition, new _utilities.ObjectStack(fragment.typeCondition), context);
      const {
        namespace
      } = context;
      return [...statements, ...(namespace ? [t.exportNamedDeclaration(namespace, [])] : []), t.exportNamedDeclaration(t.tsInterfaceDeclaration(t.identifier(context.typeName), null, null, body), [])];
    }, []);
    const {
      schemaImports
    } = file;

    if (schemaImports) {
      fileBody.unshift(schemaImports);
    }

    return generate(t.file(t.program(fileBody), [], [])).code;
  }

  const context = new _context.OperationContext(operation, ast, options, file);
  const partialContext = new _context.OperationContext(operation, ast, { ...options,
    partial: true
  }, file);
  let rootType;

  if (operation.operationType === _graphqlToolUtilities.OperationType.Query) {
    rootType = ast.schema.getQueryType();
  } else if (operation.operationType === _graphqlToolUtilities.OperationType.Mutation) {
    rootType = ast.schema.getMutationType();
  } else {
    rootType = ast.schema.getSubscriptionType();
  }

  const variables = operation.variables.filter(_graphqlToolUtilities.isTypedVariable).length > 0 ? context.export((0, _language.variablesInterface)(operation.variables, context)) : null;
  const operationInterface = t.tsInterfaceDeclaration(t.identifier(context.typeName), null, null, (0, _language.tsInterfaceBodyForObjectField)(operation, rootType, new _utilities.ObjectStack(rootType), context));
  const operationPartialInterface = t.tsInterfaceDeclaration(t.identifier(partialContext.typeName), null, null, (0, _language.tsInterfaceBodyForObjectField)(operation, rootType, new _utilities.ObjectStack(rootType), partialContext));
  const {
    schemaImports
  } = file;
  const {
    namespace
  } = context;
  const {
    namespace: partialNamespace
  } = partialContext;
  const {
    exportFormat = _types2.ExportFormat.Document
  } = options;
  const documentType = exportFormat === _types2.ExportFormat.Document ? 'DocumentNode' : 'SimpleDocument';
  const documentNodeImport = t.importDeclaration([t.importSpecifier(t.identifier(documentType), t.identifier(documentType))], t.stringLiteral('graphql-typed'));
  const documentNodeDeclaratorIdentifier = t.identifier('document');
  documentNodeDeclaratorIdentifier.typeAnnotation = t.tsTypeAnnotation(t.tsTypeReference(t.identifier(documentType), t.tsTypeParameterInstantiation([t.tsTypeReference(t.identifier(context.typeName)), variables || t.tsNeverKeyword(), t.tsTypeReference(t.identifier(partialContext.typeName))])));
  const documentNodeDeclaration = t.variableDeclaration('const', [t.variableDeclarator(documentNodeDeclaratorIdentifier)]);
  documentNodeDeclaration.declare = true;
  const documentNodeExport = t.exportDefaultDeclaration(t.identifier('document'));
  const fileBody = [documentNodeImport];

  if (schemaImports) {
    fileBody.push(schemaImports);
  }

  if (partialNamespace) {
    fileBody.push(t.exportNamedDeclaration(partialNamespace, []));
  }

  fileBody.push(t.exportNamedDeclaration(operationPartialInterface, []));

  if (namespace) {
    fileBody.push(t.exportNamedDeclaration(namespace, []));
  }

  fileBody.push(t.exportNamedDeclaration(operationInterface, []), documentNodeDeclaration, documentNodeExport);
  return generate(t.file(t.program(fileBody), [], [])).code;
}