"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.tsInterfaceBodyForObjectField = tsInterfaceBodyForObjectField;
exports.variablesInterface = variablesInterface;

var t = _interopRequireWildcard(require("@babel/types"));

var _graphql = require("graphql");

var _graphqlToolUtilities = require("graphql-tool-utilities");

var _utilities = require("../utilities.js");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function tsInterfaceBodyForObjectField({
  fields = []
}, graphQLType, stack, context, requiresTypename = false) {
  const {
    schema
  } = context.ast;
  const isRootType = !Array.isArray(graphQLType) && (graphQLType === schema.getQueryType() || graphQLType === schema.getMutationType() || graphQLType === schema.getSubscriptionType());
  const uniqueFields = fields.filter(field => {
    if (stack.hasSeenField(field)) {
      return false;
    }

    stack.sawField(field);
    return true;
  });
  const typenameField = {
    fieldName: '__typename',
    responseName: '__typename',
    type: new _graphql.GraphQLNonNull(_graphql.GraphQLString),
    isConditional: false
  };
  const typename = (context.options.addTypename && !isRootType || requiresTypename) && !stack.hasSeenField(typenameField) ? tsPropertyForField(typenameField, graphQLType, stack, context, requiresTypename) : null;
  const body = uniqueFields.map(field => tsPropertyForField(field, graphQLType, stack, context, requiresTypename));
  return t.tsInterfaceBody(typename ? [typename, ...body] : body);
}

function tsTypeForInlineFragment(inlineFragment, _graphQLType, stack, context, requiresTypename = false) {
  const {
    typeCondition
  } = inlineFragment;
  const interfaceDeclaration = t.tsInterfaceDeclaration(t.identifier(stack.name), null, null, tsInterfaceBodyForObjectField(inlineFragment, typeCondition, stack, context, requiresTypename));
  return context.export(interfaceDeclaration);
}

function tsTypeForObjectField(field, graphQLType, stack, context) {
  const {
    inlineFragments = []
  } = field;
  const possibleTypes = (0, _graphql.isInterfaceType)(graphQLType) || (0, _graphql.isUnionType)(graphQLType) ? context.ast.schema.getPossibleTypes(graphQLType) : [];

  if (inlineFragments.length) {
    const fragmentTypes = [...inlineFragments].map(inlineFragment => tsTypeForInlineFragment(inlineFragment, graphQLType, stack.fragment(inlineFragment.typeCondition), context, context.options.partial));
    const typesCoveredByInlineFragments = new Set([...inlineFragments].reduce((types, inlineFragment) => [...types, ...inlineFragment.possibleTypes], []));
    const missingPossibleTypes = possibleTypes.filter(possibleType => {
      return !typesCoveredByInlineFragments.has(possibleType);
    });
    const otherStack = stack.fragment();
    const otherTypeInterface = t.tsInterfaceDeclaration(t.identifier(otherStack.name), null, null, tsInterfaceBodyForObjectField(field, missingPossibleTypes, otherStack, context, context.options.partial));
    const otherType = context.export(otherTypeInterface);
    return t.tsUnionType([...fragmentTypes, otherType]);
  } else if (possibleTypes.length === 1) {
    // When we have an interface or union type, but no inline fragments, it
    // means that there is only one conforming type for the union/ interface.
    // Here, we construct a "nothing" type that stands in for future additions
    // to the membership of the union/ interface.
    const otherStack = stack.fragment();
    const otherTypeInterface = t.tsInterfaceDeclaration(t.identifier(otherStack.name), null, null, tsInterfaceBodyForObjectField({
      fields: []
    }, [], otherStack, context, context.options.partial));
    const interfaceStack = stack.fragment(possibleTypes[0]);
    const interfaceDeclaration = t.tsInterfaceDeclaration(t.identifier(interfaceStack.name), null, null, tsInterfaceBodyForObjectField(field, graphQLType, interfaceStack, context));
    return t.tsUnionType([context.export(interfaceDeclaration), context.export(otherTypeInterface)]);
  }

  const interfaceDeclaration = t.tsInterfaceDeclaration(t.identifier(stack.name), null, null, tsInterfaceBodyForObjectField(field, graphQLType, stack, context));
  return context.export(interfaceDeclaration);
}

function tsTypenameForGraphQLType(type) {
  return t.tsLiteralType(t.stringLiteral(type.name));
}

function tsPropertyForField(field, parentType, stack, context, isRequiredTypename = false) {
  if (field.fieldName === '__typename' && parentType) {
    const optional = !isRequiredTypename && (context.options.partial || field.isConditional || !(0, _graphql.isNonNullType)(field.type));
    const parentTypes = Array.isArray(parentType) ? parentType : [parentType];
    const allPossibleTypes = parentTypes.reduce((all, type) => [...all, ...((0, _graphql.isAbstractType)(type) ? context.ast.schema.getPossibleTypes(type) : [type])], []);
    let typename;

    if (allPossibleTypes.length === 0) {
      typename = t.tsLiteralType(t.stringLiteral(''));
    } else if (allPossibleTypes.length > 1) {
      typename = t.tsUnionType(allPossibleTypes.map(tsTypenameForGraphQLType));
    } else {
      typename = tsTypenameForGraphQLType(allPossibleTypes[0]);
    }

    const typenameProperty = t.tsPropertySignature(t.identifier(field.responseName), optional ? t.tsTypeAnnotation(t.tsUnionType([typename, t.tsNullKeyword()])) : t.tsTypeAnnotation(typename));
    typenameProperty.optional = optional;
    return typenameProperty;
  }

  const property = t.tsPropertySignature(t.identifier(field.responseName), t.tsTypeAnnotation(tsTypeForGraphQLType(field.type, field, stack, context)));
  property.optional = context.options.partial || field.isConditional || !(0, _graphql.isNonNullType)(field.type);
  return property;
}

function tsTypeForGraphQLType(graphQLType, field, stack, context) {
  let type;
  const forceNullable = context.options.partial || field.isConditional && graphQLType === field.type;
  const isNonNull = !forceNullable && (0, _graphql.isNonNullType)(graphQLType);
  const unwrappedGraphQLType = (0, _graphql.isNonNullType)(graphQLType) ? graphQLType.ofType : graphQLType;

  if ((0, _graphql.isScalarType)(unwrappedGraphQLType)) {
    if (Object.prototype.hasOwnProperty.call(_utilities.scalarTypeMap, unwrappedGraphQLType.name)) {
      type = _utilities.scalarTypeMap[unwrappedGraphQLType.name];
    } else {
      context.file.import(unwrappedGraphQLType.name);
      type = t.tsTypeReference(t.identifier(unwrappedGraphQLType.name));
    }
  } else if ((0, _graphql.isEnumType)(unwrappedGraphQLType)) {
    context.file.import(unwrappedGraphQLType.name);
    type = t.tsTypeReference(t.identifier(unwrappedGraphQLType.name));
  } else if ((0, _graphql.isListType)(unwrappedGraphQLType)) {
    const {
      ofType
    } = unwrappedGraphQLType;
    const arrayType = tsTypeForGraphQLType(ofType, field, stack, context);
    type = t.tsArrayType(t.isTSUnionType(arrayType) ? t.tsParenthesizedType(arrayType) : arrayType);
  } else if ((0, _graphql.isObjectType)(unwrappedGraphQLType) || (0, _graphql.isInterfaceType)(unwrappedGraphQLType) || (0, _graphql.isUnionType)(unwrappedGraphQLType)) {
    type = tsTypeForObjectField(field, unwrappedGraphQLType, stack.nested(field, unwrappedGraphQLType), context);
  } else {
    type = t.tsAnyKeyword();
  }

  return isNonNull ? type : t.tsUnionType([type, t.tsNullKeyword()]);
}

function variablesInterface(variables, context) {
  return t.tsInterfaceDeclaration(t.identifier('Variables'), null, null, t.tsInterfaceBody(variables.filter(_graphqlToolUtilities.isTypedVariable).map(variable => tsPropertyForVariable(variable, context))));
}

function tsPropertyForVariable({
  name,
  type
}, context) {
  const property = t.tsPropertySignature(t.identifier(name), t.tsTypeAnnotation(tsTypeForGraphQLInputType(type, context)));
  property.optional = !(0, _graphql.isNonNullType)(type);
  return property;
}

function tsTypeForGraphQLInputType(graphQLType, context) {
  let type;
  const unwrappedGraphQLType = (0, _graphql.isNonNullType)(graphQLType) ? graphQLType.ofType : graphQLType;

  if ((0, _graphql.isScalarType)(unwrappedGraphQLType)) {
    if (Object.prototype.hasOwnProperty.call(_utilities.scalarTypeMap, unwrappedGraphQLType.name)) {
      type = _utilities.scalarTypeMap[unwrappedGraphQLType.name];
    } else {
      context.file.import(unwrappedGraphQLType.name);
      type = t.tsTypeReference(t.identifier(unwrappedGraphQLType.name));
    }
  } else if ((0, _graphql.isEnumType)(unwrappedGraphQLType) || (0, _graphql.isInputObjectType)(unwrappedGraphQLType)) {
    context.file.import(unwrappedGraphQLType.name);
    type = t.tsTypeReference(t.identifier(unwrappedGraphQLType.name));
  } else {
    const {
      ofType
    } = unwrappedGraphQLType;
    const arrayType = tsTypeForGraphQLInputType(ofType, context);
    type = t.tsArrayType((0, _graphql.isNonNullType)(ofType) ? arrayType : t.tsParenthesizedType(arrayType));
  }

  return (0, _graphql.isNonNullType)(graphQLType) ? type : t.tsUnionType([type, t.tsNullKeyword()]);
}