"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ObjectStack = void 0;

var _upperCaseFirst = require("upper-case-first");

class ObjectStack {
  get name() {
    const {
      parent,
      field,
      isFragment,
      type
    } = this;
    const fieldName = field ? (0, _upperCaseFirst.upperCaseFirst)(field.responseName) : '';
    const name = `${parent ? parent.name : ''}${fieldName}`;
    return isFragment ? `${name}${type ? type.name : 'Other'}` : name;
  }

  constructor(type, field, parent, isFragment = false) {
    this.type = type;
    this.field = field;
    this.parent = parent;
    this.isFragment = isFragment;
    this.seenFields = new Set();
  }

  nested(field, type) {
    return new ObjectStack(type, field, this);
  }

  fragment(type) {
    return new ObjectStack(type, this.field, this.parent, true);
  }

  sawField(field) {
    this.seenFields.add(field.responseName);
  }

  hasSeenField(field) {
    return this.seenFields.has(field.responseName);
  }

}

exports.ObjectStack = ObjectStack;