import { watch } from 'chokidar';
import { getGraphQLProjectIncludedFilePaths, getGraphQLProjects, getGraphQLSchemaPaths, resolvePathRelativeToConfig, } from 'graphql-config-utilities';
import { AbstractGraphQLFilesystem } from './graphql-filesystem';
export class DefaultGraphQLFilesystem extends AbstractGraphQLFilesystem {
    constructor() {
        super(...arguments);
        this.watchers = [];
    }
    async watch(config) {
        this.watchers.push(...this.setupDocumentWatchers(config).concat(this.setupSchemaWatcher(config)));
        await Promise.all(this.watchers.map(watcher => new Promise(resolve => watcher.on('ready', () => resolve()))));
    }
    dispose() {
        this.watchers.forEach(watcher => {
            watcher.close();
        });
        this.watchers.length = 0;
    }
    getGraphQLProjectIncludedFilePaths(projectConfig) {
        return getGraphQLProjectIncludedFilePaths(projectConfig);
    }
    setupDocumentWatchers(config) {
        return getGraphQLProjects(config)
            .filter(({ includes }) => includes.length > 0)
            .map(projectConfig => {
            return watch(projectConfig.includes.map(include => resolvePathRelativeToConfig(projectConfig, include)), {
                ignored: projectConfig.excludes.map(exclude => resolvePathRelativeToConfig(projectConfig, exclude)),
                ignoreInitial: true,
            })
                .on('add', (filePath) => this.emit('change:document', filePath, projectConfig))
                .on('change', (filePath) => this.emit('change:document', filePath, projectConfig))
                .on('unlink', (filePath) => this.emit('delete:document', filePath, projectConfig));
        });
    }
    setupSchemaWatcher(config) {
        return watch(getGraphQLSchemaPaths(config), {
            ignoreInitial: true,
        }).on('change', schemaPath => this.emit('change:schema', schemaPath));
    }
}
